#include "DHT.h"
#include "lcd.h"

#define DHT_PIN 40
#define DHT_TYPE DHT11

DHT dht(DHT_PIN, DHT_TYPE);

LGFX lcd;

float toRadians(float degrees) {
  return degrees*PI/180.0;
}

float fmap(float x, float in_min, float in_max, float out_min, float out_max) {
  return (x - in_min) * (out_max - out_min) / (in_max - in_min) + out_min;
}

void drawDial(LGFX& gfx, float x, float y, float r, float degrees, int color) {
  
  float angle = toRadians(degrees);
  
  float xe = x + r*cos(angle);
  float ye = y + r*sin(angle);

  gfx.drawCircle(x, y, 5, color);
  gfx.drawCircle(x, y, r+2, color);
  gfx.drawLine(x, y, xe, ye, color);

}

void setup() {
  Serial.begin(9600);
  Serial.println("ESP32 Terminal Weather Station");

  dht.begin();

  lcd.init();

  pinMode(LCD_BLK, OUTPUT);
  digitalWrite(LCD_BLK, HIGH);

  lcd.setRotation(3);
}

void loop() {
  
  float humidity = dht.readHumidity();
  float temperature = dht.readTemperature();

  if (isnan(humidity) || isnan(temperature)) {
    Serial.println("Failed to read data from sensor.");
    return;
  }

  float heatIndex = dht.computeHeatIndex(temperature, humidity, false);
  
  lcd.fillScreen(TFT_BLACK);
  lcd.setCursor(200, 10);
  lcd.println("WEATHER STATION");

  lcd.setCursor(0, 310);
  lcd.print("hmdt: ");
  lcd.print(humidity);
  lcd.print("%; temp: ");
  lcd.print(temperature);
  lcd.print("C; hidx: ");
  lcd.print(heatIndex);
  lcd.print("C");

  float startAngle = 135;
  float endAngle = 360+45;

  float temperatureAngle = fmap(temperature, 0, 50, startAngle, endAngle);
  drawDial(lcd, 1*480/4, 160, 50, temperatureAngle, TFT_WHITE);
  lcd.setCursor(1*480/4-5*8, 90);
  lcd.print("temperature");
  lcd.setCursor(1*480/4-3*8, 220);
  lcd.print(temperature); lcd.print("C");


  float humidityAngle = fmap(humidity, 20, 90, startAngle, endAngle);
  drawDial(lcd, 2*480/4, 160, 50, humidityAngle, TFT_WHITE);
  lcd.setCursor(2*480/4-4*8, 90);
  lcd.print("humidity");
  lcd.setCursor(2*480/4-3*8, 220);
  lcd.print(humidity); lcd.print("%");


  float heatIndexAngle = fmap(heatIndex, 0, 50, startAngle, endAngle);
  drawDial(lcd, 3*480/4, 160, 50, heatIndexAngle, TFT_WHITE);
  lcd.setCursor(3*480/4-5*8, 90);
  lcd.print("heat index");
  lcd.setCursor(3*480/4-3*8, 220);
  lcd.print(heatIndex); lcd.print("C");

  delay(2000);
 
}
